﻿ package com.nosuch {
 	import flash.events.*;
 	import flash.net.XMLSocket;
 	import flash.utils.*;
 	
 	import onyx.core.Console;
 	import onyx.errors.*;
 	import onyx.events.*;
 	import onyx.file.*;
 	import onyx.midi.*;

	public class NthCmdClient extends NthPersistentClient {
		
		private static var instance:NthCmdClient = new NthCmdClient();
		private var _tosend:String;
		private var _path:String;

		public function NthCmdClient() {
			super();
			if ( instance ) {
				throw INVALID_CLASS_CREATION;
			}
		}
		
		public static function getInstance():NthCmdClient {
			trace("NthCmdClient.getInstance called, = ",instance);
			return instance;
		}
		
		public function listMidiInputs():void {
	     	_sendXML("<list_midi_inputs/>");
		}
		
		public function listMidiOutput():void {
	     	_sendXML("<list_midi_outputs/>");
		}
		
		public function writeMidi(m:MidiMsg, deviceIndex:int = -1):void {
			var x:XML = m.toXML()
			// Ideally, this should add a 'deviceindex=#' attribute when
			// deviceIndex is specified.
	     	var s:String = "<write_midi>"+x.toXMLString()+"</write_midi>";
	     	_sendXML(s);
		}
		
		public function readFile(path:String):void {
			_path = path;
	     	_sendXML("<read_file><name>"+path+"</name></read_file>");
		}
		
		public function writeFile(path:String, data:String):void {
			_path = path;
	     	_sendXML("<write_file><name>"+path+"</name><data>"+escape(data)+"</data></write_file>");
		}
		
		public function writeFileBytes(path:String, bytes:ByteArray):void {
			_path = path;
			var writedata:String = "";
			bytes.position = 0;
			while ( bytes.bytesAvailable > 0 ) {
				var s:String = String.fromCharCode(bytes.readByte());
				writedata = writedata.concat(escape(s));
			}
	     	_sendXML("<write_file><name>"+path+"</name><data>"+writedata+"</data></write_file>");
		}
		
		public function listDir(path:String):void {
			_path = path;
	     	_sendXML("<list_dir><name>"+path+"</name></list_dir>");
		}
		
		public function shutdown():void {
	     	_sendXML("<quickshutdown/>",false);
		}
		private function _sendXML(s:String, queueit:Boolean=true):void {
			// trace("_sendXML SENDING s=",s," isconnected=",isConnected());
			if ( isConnected() ) {
		     	this.send(s);
			} else if ( queueit ) {
				_tosend = s;
			}
		}
		
	    override public function connectHandler(event:Event):void {
	    	super.connectHandler(event);
	    	Console.output("NthCmdClient is connected to NthServer...");
	    	if ( _tosend != null ) {
				trace("connectHandler SENDING s=",_tosend);
		     	this.send(_tosend);
		    }
	    }
	    
		override public function closeHandler(event:Event):void {
			Console.output("NthCmdClient has been disconnected...");
			trace("NthCmdClient closehandler called");
			super.closeHandler(event);
		}

		override public function dataHandler(event:DataEvent):void {
			
			var client:NthCmdClient = event.currentTarget as NthCmdClient;
			var path:String = client._path;
			
	        var x:XML = new XML(event.data);
	        
	       	// A read_file will produce <file_contents>
	       	// A write_file will produce an <ok>
	       	// A list_dir will produce a <directory>
	       	// A write_midi will produce an <ok>
	       	// An error will produce a <error>
	       	
	        var e:NthCmdEvent;
	        var fe:NthFileEvent;
	        
	        if ( x.localName() == "error" ) {
	        	e = new NthCmdEvent(NthCmdEvent.DONE);
	        	e.error = x.toString();
	        	dispatchEvent(e);
	        	return;
	        }
	        
	        if ( x.localName() == "ok" ) {
	        	e = new NthCmdEvent(NthCmdEvent.DONE);
	        	dispatchEvent(e);
	        	return;
	        }
	        
	        if ( x.localName() == "file_contents" ) {
	        	fe = new NthFileEvent(NthCmdEvent.DONE,x);
	        	fe.path = path;
				fe.fileData = unescape(x.toString());
	        	dispatchEvent(fe);
				return;
	        }
	        
	        if ( x.localName() == "directory" ) {
	        	fe = new NthFileEvent(NthCmdEvent.DONE,x);
	        	fe.path = path;
				// create the Folder
				var list:FolderList = new FolderList(path);
				for each (var f:XML in x.*) {
					var name:String = f.toString().toLowerCase();
					if ( f.localName() == "file" ) {
						list.files.push(new File(path + name, ''));
					} else if ( f.localName() == "dir" ) {
						if (name === '..') {
							name = path.substr(0, path.lastIndexOf('/', path.length - 2)) + '/';
						} else {
							name = path + "/" + name;
						}
						list.folders.push(new Folder(name));
					}
				}
				fe.folderList = list;
				dispatchEvent(fe);
				return;
			}
			
        	e = new NthCmdEvent(NthCmdEvent.DONE);
	       	e.error = "Unknown NthCmdClient response: "+x.localName();
	       	dispatchEvent(e);
	    }
	}
}