﻿ package com.nosuch {
 	import flash.events.*;
 	import flash.net.*;
 	import flash.utils.*;
 	
	public class NthEventClient extends NthPersistentClient {
		
		private static var instance:NthEventClient	= new NthEventClient();
		private static var fingers:Object		= new Object();
		
		private var _first_event_nth_millisecs:int = -1;
		private var _first_event_flash_millisecs:int;
		private var _first_event_dt:int;
		private var _numdown:int = 0;
		
		public function NthEventClient() {
			super();
			// We only want one of these to be active - it sits waiting for
			// the next event from the NthServer, and as soon as an event comes in,
			// it requests the next one.
			if ( instance ) {
				throw INVALID_CLASS_CREATION;
			}
		}
		
		public static function getInstance():NthEventClient {
			return instance;
		}
		
	    override public function connectHandler(event:Event):void {
        	super.connectHandler(event);
       		var me:NthEventClient = event.currentTarget as NthEventClient;
        	trace("NthEventClient is connected to NthServer...");
        	me._numdown = 0;
       		me._first_event_nth_millisecs = -1;
	        requestNextEvent();
	    }
	    
	    override public function requestNextEvent(overloaded:Boolean = false):void {
	    	if ( overloaded ) {
		     	this.send("<next_event overloaded=\"true\"/>");
		    } else {
		     	this.send("<next_event/>");
		    }
	    }
	
		public function dataHandler(event:DataEvent):void {
			var overloaded:Boolean = false;
	        var x:XML = new XML(event.data);
       		var me:NthEventClient = event.currentTarget as NthEventClient;
       		if ( x.localName() == "reset" ) {
       			trace("GOT RESET from NTHSERVER");
        		me._first_event_nth_millisecs = -1;
	        	_numdown = 0;
       		} else if ( x.localName() == "event" ) {
	        	var tm:Number = x.attribute("time");
	        	var c:XMLList = x.children()
	        	for each ( var x2:XML in c ) {
	        		var s:String = x2.localName();
	        		// Ideally, we would check the bytesAvailable of
	        		// the Socket, to see if we're getting behind,
	        		// but XMLSocket doesn't have bytesAvailable!
	        		// So, we look at the times.
	        		if ( me._first_event_nth_millisecs < 0 ) {
	        			me._first_event_nth_millisecs = tm * 1000;
		        		me._first_event_flash_millisecs = getTimer();
		        		me._first_event_dt = me._first_event_flash_millisecs - me._first_event_nth_millisecs;
		        		trace("FIRST_EVENT_DT = ",me._first_event_dt);
		        		trace("first_event_flash_millisecs = ",me._first_event_flash_millisecs);
		        		trace("first_event_nth_millisecs = ",me._first_event_nth_millisecs);
	        		}
	        		var dt_flash:int = getTimer() - me._first_event_flash_millisecs;
	        		var dt_nth:int = tm*1000 - me._first_event_nth_millisecs;
	        		var dt_this:int = getTimer() - tm*1000;
	        		var dt_dt:int = dt_this - me._first_event_dt;
	        		if ( s.search("finger_") == 0 ) {
	        			var ftype:String = s.substr(7);
	        			if ( ftype == "drag" && dt_dt > 500 ) {
	        				overloaded = true;
	        				// trace("Skipping finger drag event, dt=",dt_dt," dt_flash=",dt_flash," dt_this=",dt_this);
	        				// trace("     me.first_event_dt=",me._first_event_dt);
		        			var e:NthStatusEvent = new NthStatusEvent(NthStatusEvent.OVERLOADED);
		        			dispatchEvent(e);
	        				// trace("FINGER SETTING OVERLOADED TO TRUE, OVERLOADED EVENT!");
	        				// continue;
	        			}
	        			var f:FingerEvent = new FingerEvent(ftype,tm,x2);
	        			updateFingers(f);
	        			dispatchEvent(f);
	        		} else if ( s.search("midi_") == 0 ) {
	        			var mtype:String = s.substr(5);
	        			var m:MidiEvent = new MidiEvent(mtype,tm,x2);
	        			if ( dt_dt > 500 ) {
	        				overloaded = true;
		        			var e2:NthStatusEvent = new NthStatusEvent(NthStatusEvent.OVERLOADED);
		        			dispatchEvent(e2);
	        				// trace("MIDI SETTING OVERLOADED TO TRUE, OVERLOADED EVENT");
	        				
	        				var ch:int = 0;
	        				if ( m.midimsg is MidiNoteOn ) {
		        				var nton:MidiNoteOn = m.midimsg as MidiNoteOn;
		        				ch = nton.channel;
		        			} else if ( m.midimsg is MidiNoteOff ) {
		        				var ntoff:MidiNoteOff= m.midimsg as MidiNoteOff;
		        				ch = ntoff.channel;
	        				}
	        				if ( ! ( ch == 0 || ch == 16) ) {
		        				// trace("Skipping non-control midi, dt is ",dt_dt," dt_this=",dt_this," first=",me._first_event_dt," ch=",ch);
		        				continue;
		        			}
	        			}
	        			// trace("NthEvent s=",s," m=",m," m.midimsg=",m.midimsg);
	        			dispatchEvent(m);
	        		}
	        	}
	        }
	       	requestNextEvent(overloaded);
	    }
	    
		override public function closeHandler(event:Event):void {
			Console.output("NthEventClient has been disconnected...");
        	_numdown = 0;
			super.closeHandler(event);
		}
	     
	    private function updateFingers(f:FingerEvent):void {
	    	var uid:String = f.fingerUID();
			var oldfs:FingerState = fingers[uid];
			var newfs:FingerState = new FingerState(f);
			if ( f.type == FingerEvent.DOWN ) {
				newfs.downorder = _numdown;
				_numdown++;
			} else if ( f.type == FingerEvent.UP ) {
				if ( oldfs == null ) {
					// If we get an UP event without any record of
					// a previous up/down event, ignore it.
					return;
				}
				_numdown--
				newfs.downorder = -1;
				for each (var fs:FingerState in fingers) {
					if ( fs != oldfs && fs.downorder > oldfs.downorder ) {
						fs.downorder--;
					}
				}
			} else { // i.e. DRAG
				if ( oldfs == null ) {
					// If we get a DRAG event without any record of
					// a previous up/down event, ignore it.
					return;
				}
				newfs.downorder = oldfs.downorder;
			}
			delete fingers[uid];
			fingers[uid] = newfs;
	    }
	    
	    private function printfingers():void {
	    	for each (var fs:FingerState in fingers) {
	    		trace("   fs=",fs);
	    	}
	    }
	    
	    override public function getFingerStates():Object {
			return fingers;
	    }
	}
}